@@ -32,25 +32,14 @@ class OpTest(TestBase):
         op.rename_table('t1', 't2', schema="foo")
         context.assert_("ALTER TABLE foo.t1 RENAME TO foo.t2")
 
-    def test_rename_table_postgresql(self):
-        context = op_fixture("postgresql")
-        op.rename_table('t1', 't2')
-        context.assert_("ALTER TABLE t1 RENAME TO t2")
-
-    def test_rename_table_schema_postgresql(self):
-        context = op_fixture("postgresql")
-        op.rename_table('t1', 't2', schema="foo")
-        context.assert_("ALTER TABLE foo.t1 RENAME TO t2")
-
     def test_create_index_no_expr_allowed(self):
         op_fixture()
         assert_raises_message(
             ValueError,
-            "String or text\(\) construct expected",
+            r"String or text\(\) construct expected",
             op.create_index, 'name', 'tname', [func.foo(column('x'))]
         )
 
-    @config.requirements.sqlalchemy_09
     def test_add_column_schema_hard_quoting(self):
         from sqlalchemy.sql.schema import quoted_name
         context = op_fixture("postgresql")
@@ -62,7 +51,6 @@ class OpTest(TestBase):
             'ALTER TABLE "some.schema".somename ADD COLUMN colname VARCHAR'
         )
 
-    @config.requirements.sqlalchemy_09
     def test_rename_table_schema_hard_quoting(self):
         from sqlalchemy.sql.schema import quoted_name
         context = op_fixture("postgresql")
@@ -74,7 +62,6 @@ class OpTest(TestBase):
             'ALTER TABLE "some.schema".t1 RENAME TO t2'
         )
 
-    @config.requirements.sqlalchemy_09
     def test_add_constraint_schema_hard_quoting(self):
         from sqlalchemy.sql.schema import quoted_name
         context = op_fixture("postgresql")
@@ -98,7 +85,6 @@ class OpTest(TestBase):
         context.assert_(
             'CREATE INDEX geocoded ON locations ("IShouldBeQuoted")')
 
-    @config.requirements.fail_before_sqla_080
     def test_create_index_expressions(self):
         context = op_fixture()
         op.create_index(
@@ -108,29 +94,6 @@ class OpTest(TestBase):
         context.assert_(
             "CREATE INDEX geocoded ON locations (lower(coordinates))")
 
-    @config.requirements.fail_before_sqla_080
-    def test_create_index_postgresql_expressions(self):
-        context = op_fixture("postgresql")
-        op.create_index(
-            'geocoded',
-            'locations',
-            [text('lower(coordinates)')],
-            postgresql_where=text("locations.coordinates != Null"))
-        context.assert_(
-            "CREATE INDEX geocoded ON locations (lower(coordinates)) "
-            "WHERE locations.coordinates != Null")
-
-    def test_create_index_postgresql_where(self):
-        context = op_fixture("postgresql")
-        op.create_index(
-            'geocoded',
-            'locations',
-            ['coordinates'],
-            postgresql_where=text("locations.coordinates != Null"))
-        context.assert_(
-            "CREATE INDEX geocoded ON locations (coordinates) "
-            "WHERE locations.coordinates != Null")
-
     def test_add_column(self):
         context = op_fixture()
         op.add_column('t1', Column('c1', Integer, nullable=False))
@@ -373,7 +336,7 @@ class OpTest(TestBase):
         )
 
     def test_alter_column_schema_type_unnamed(self):
-        context = op_fixture('mssql')
+        context = op_fixture('mssql', native_boolean=False)
         op.alter_column("t", "c", type_=Boolean())
         context.assert_(
             'ALTER TABLE t ALTER COLUMN c BIT',
@@ -381,7 +344,7 @@ class OpTest(TestBase):
         )
 
     def test_alter_column_schema_schema_type_unnamed(self):
-        context = op_fixture('mssql')
+        context = op_fixture('mssql', native_boolean=False)
         op.alter_column("t", "c", type_=Boolean(), schema='foo')
         context.assert_(
             'ALTER TABLE foo.t ALTER COLUMN c BIT',
@@ -389,7 +352,7 @@ class OpTest(TestBase):
         )
 
     def test_alter_column_schema_type_named(self):
-        context = op_fixture('mssql')
+        context = op_fixture('mssql', native_boolean=False)
         op.alter_column("t", "c", type_=Boolean(name="xyz"))
         context.assert_(
             'ALTER TABLE t ALTER COLUMN c BIT',
@@ -397,7 +360,7 @@ class OpTest(TestBase):
         )
 
     def test_alter_column_schema_schema_type_named(self):
-        context = op_fixture('mssql')
+        context = op_fixture('mssql', native_boolean=False)
         op.alter_column("t", "c", type_=Boolean(name="xyz"), schema='foo')
         context.assert_(
             'ALTER TABLE foo.t ALTER COLUMN c BIT',
@@ -405,7 +368,7 @@ class OpTest(TestBase):
         )
 
     def test_alter_column_schema_type_existing_type(self):
-        context = op_fixture('mssql')
+        context = op_fixture('mssql', native_boolean=False)
         op.alter_column(
             "t", "c", type_=String(10), existing_type=Boolean(name="xyz"))
         context.assert_(
@@ -414,7 +377,7 @@ class OpTest(TestBase):
         )
 
     def test_alter_column_schema_schema_type_existing_type(self):
-        context = op_fixture('mssql')
+        context = op_fixture('mssql', native_boolean=False)
         op.alter_column("t", "c", type_=String(10),
                         existing_type=Boolean(name="xyz"), schema='foo')
         context.assert_(
@@ -471,6 +434,16 @@ class OpTest(TestBase):
             "REFERENCES bar2.t2 (bat, hoho)"
         )
 
+    def test_add_foreign_key_schema_same_tablename(self):
+        context = op_fixture()
+        op.create_foreign_key('fk_test', 't1', 't1',
+                              ['foo', 'bar'], ['bat', 'hoho'],
+                              source_schema='foo2', referent_schema='bar2')
+        context.assert_(
+            "ALTER TABLE foo2.t1 ADD CONSTRAINT fk_test FOREIGN KEY(foo, bar) "
+            "REFERENCES bar2.t1 (bat, hoho)"
+        )
+
     def test_add_foreign_key_onupdate(self):
         context = op_fixture()
         op.create_foreign_key('fk_test', 't1', 't2',
@@ -690,7 +663,6 @@ class OpTest(TestBase):
             "CREATE UNIQUE INDEX ik_test ON t1 (foo, bar)"
         )
 
-    @config.requirements.fail_before_sqla_09
     def test_create_index_quote_flag(self):
         context = op_fixture()
         op.create_index('ik_test', 't1', ['foo', 'bar'], quote=True)
@@ -868,7 +840,6 @@ class OpTest(TestBase):
         op.drop_constraint("f1", "t1", type_="foreignkey")
         context.assert_("ALTER TABLE t1 DROP FOREIGN KEY f1")
 
-    @config.requirements.fail_before_sqla_084
     def test_naming_changes_drop_idx(self):
         context = op_fixture('mssql')
         op.drop_index('ik_test', tablename='t1')
@@ -876,7 +847,6 @@ class OpTest(TestBase):
 
 
 class SQLModeOpTest(TestBase):
-    @config.requirements.sqlalchemy_09
     def test_auto_literals(self):
         context = op_fixture(as_sql=True, literal_binds=True)
         from sqlalchemy.sql import table, column
